import 'dart:convert';
import 'dart:io';

import 'package:dio/dio.dart';
import 'package:yuro/yuro_core/yuro_core.dart';
import 'package:yuro/yuro_util/yuro_util.dart';

import 'dio_cache.dart';
import 'dio_cache_options.dart';

class DioCacheInterceptor extends Interceptor {
  static Future<DioCacheInterceptor> init(DioCacheOptions options) async {
    Yuro.registerHiveAdapter(DioCacheAdapter());
    final box = await Yuro.openHiveBox<DioCache>();
    // 清理过期的缓存
    final staleList = box.values.where((element) => element.isStaled).map((e) => e.storeKey);
    if (staleList.isNotEmpty) box.deleteAll(staleList);
    return DioCacheInterceptor._(options, box);
  }

  DioCacheInterceptor._(this._cacheOptions, this._cacheBox);

  final DioCacheOptions _cacheOptions;
  final Box<DioCache> _cacheBox;

  @override
  void onRequest(RequestOptions options, RequestInterceptorHandler handler) async {
    if (_shouldCache(options)) {
      final key = _cacheOptions.keyBuilder(options);
      final cache = _cacheBox.get(key);
      if (cache != null && !cache.isExpires) {
        //
      }
      options.extra[DioCache.REQUEST_SEND_DATE] = DateTime.now();
    }
    handler.next(options);
  }

  @override
  void onResponse(Response response, ResponseInterceptorHandler handler) async {
    if (_shouldCache(response.requestOptions)) {
      final cache = DioCache.fromResponse(_cacheOptions, response);
      await _cacheBox.put(cache.storeKey, cache);
    }
    handler.next(response);
  }

  // 获取请求是否需要缓存,包括请求方式和是否配置了不缓存
  bool _shouldCache(RequestOptions options) {
    return options.method == 'GET';
  }
}
